import com.ibm.db2.app.*;
import java.sql.*;
import java.math.BigDecimal;

/*
  Class:  ExtDB

  Desc:   Demonstrate ability to query
          external data with Java/SQL.

          Retrieve data from external
          database (SQL Server 2000)
          table via JDBC for use by
          a UDTF.
 
  Author: Michael Sansoterra

  Date:   08/22/03

  COMPILE NOTES (from inside QSHELL (QSH)):

  The class file for the routine must be placed in 
  "/qibm/userdata/os400/sqllib/function"
 
  javac -d /qibm/userdata/os400/sqllib/function /myfolder/ExtDB.java
  


  TO OPTIMIZE:
  CRTJVAPGM CLSF('/qibm/userdata/os400/sqllib/function/ExtDB.class')
            OPTIMIZE(40)



  POTENTIAL ERROR:
  Program does not run properly in interactive SQL (STRSQL) when
  job's CCSID is set to 65535.  Set job's CCSID to 37 (US EBCDIC).


  Installing MS SQL Server 2000 JAR Files:
  =====================================================
  CALL sqlj.install_jar('file:/myfolder/msbase.jar',
                        'prodlib.msbase',0)

  CALL sqlj.install_jar('file:/myfolder/msutil.jar',
                        'prodlib.msutil',0)

  CALL sqlj.install_jar('file:/myfolder/mssqlserver.jar',
                        'prodlib.mssqlserver',0)
 
 
*/

public class ExtDB extends UDF {
  Connection conn;
  Statement  stmt;
  ResultSet  rs;

  // This method will be invoked by DB Manager
  public void orders(
    
    // input parameter(s)

    String parmWhere,

    // output columns

    int orderID, String customerID, 
    int employeeID, String orderDate, 
    String requiredDate, String shippedDate,
    int shipVia, BigDecimal freight,
    String shipName, String shipAddress,
    String shipCity, String shipRegion,
    String shipPostalCode, String shipCountry) throws Exception {


    // getCallType() method controls program flow

    int callType=getCallType();
    switch(callType) {

      // SQLUDF_TF_FIRST
      // invoked if UDTF definition has FINAL CALL specified 
      case SQLUDF_TF_FIRST:

        // Verify the SQL Server JDBC driver is available
        // if not, verify the sqlj.install_jar has been
        // done properly.
        try {
          Class.forName("com.microsoft.jdbc.sqlserver.SQLServerDriver");
        }
        catch (Exception e) {
          setSQLstate("38I01");
          setSQLmessage(e.getMessage());
        }
        break;

      // SQLUDF_TF_OPEN
      // invoked only once -- initialization goes here
      case SQLUDF_TF_OPEN:
        try {
          // Connect to SQL Server -- set your server's
          // URL, port, user name and password here.
          conn = DriverManager.getConnection(
                 "jdbc:microsoft:sqlserver://myserver:1433;User=nwind;Password=test;");
          stmt = conn.createStatement();
        }
        catch (Exception e) {
          setSQLstate("38I02");
          setSQLmessage(e.getMessage());
        }

        try {
          // Submit query to SQL Server database
          String sql="SELECT * FROM Northwind.DBO.Orders " +
                     (parmWhere.equals("")?"":"WHERE " + parmWhere) + 
                     " ORDER BY OrderID";

          rs = stmt.executeQuery(sql);
        }
        catch (Exception e) {
          stmt.close();
          conn.close();
          setSQLstate("38I03");
          setSQLmessage(e.getMessage());
        }
        break;

      // SQLUDF_TF_FETCH
      // invoked one or more times until SQL State is set
      // to NO DATA ("02000")
      case SQLUDF_TF_FETCH:
        if (rs.next()) {

          // set() method returns data
          // for a given UDTF output column.
          // If set() isn't called for a column,
          // the column will be NULL.

          // For primitive data types, the
          // NULL test must be done manually
          // using the wasNull() method of
          // the ResultSet() class.

          orderID=rs.getInt("OrderID");
          set(2, orderID);

          customerID=rs.getString("CustomerID");
          set(3, customerID);

          employeeID=rs.getInt("EmployeeID");
          if (!rs.wasNull())
            set(4, employeeID);

          if (rs.getDate("OrderDate")!=null)
              set(5, rs.getDate("orderDate").toString());

          if (rs.getDate("requiredDate")!=null)
              set(6, rs.getDate("requiredDate").toString());

          if (rs.getDate("shippedDate")!=null)
              set(7, rs.getDate("shippedDate").toString());

          shipVia=rs.getInt("shipVia");
          if (!rs.wasNull())
              set(8, shipVia);

          freight=rs.getBigDecimal("freight");
          set(9, freight);

          shipName=rs.getString("shipName");
          set(10, shipName);

          shipAddress=rs.getString("shipAddress");
          set(11, shipAddress);

          shipCity=rs.getString("shipCity");
          set(12, shipCity);

          shipRegion=rs.getString("shipRegion");
          set(13, shipRegion);

          shipPostalCode=rs.getString("shipPostalCode");
          set(14, shipPostalCode);

          shipCountry=rs.getString("shipCountry");
          set(15, shipCountry);

        } else {
          setSQLstate("02000");
        }
        break;

      // SQLUDF_TF_CLOSE
      // invoked after SQL State '02000' received
      // cleanup done here
      case SQLUDF_TF_CLOSE:
        rs.close();
        stmt.close();
        conn.close();

      // SQLUDF_TF_FINAL
      // invoked after SQLUDF_TF_CLOSE if UDTF 
      // definition has FINAL CALL specified.
      // Additional cleanup can be done here
      case SQLUDF_TF_FINAL:
        break;
      default:
        setSQLstate("38I04");
        setSQLmessage("Unknown call type");        
    }  // End switch
  }    // End Orders method
  
  //
  // This method is overridden to limit the number
  // of characters returned by an exception
  // to 69 characters.  setSQLmessage accepts
  // a maximum of 69.
  //
  public void setSQLmessage(String message) throws Exception {
    int msgLen=message.length();
    String tmpMessage=message.substring(0, (msgLen>69?69:msgLen));
    super.setSQLmessage(tmpMessage);
  }    // End setSQLmessage method
}      // End ExtDB class
